//--------------------------------------------------------------//
// Morphing Effect
//--------------------------------------------------------------//

//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

// transformation matrices
float4x4 mtxWorld	: WORLD;  
float4x4 mtxView	: VIEW;
float4x4 mtxProj	: PROJECTION;
float4   gLight; 
float4   gViewVec; 
float    weight;

texture baseTex;

sampler2D baseTex0 = sampler_state
{
	Texture = <baseTex>;
	MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------

struct VS_INPUT
{
	float3 Pos0      : POSITION0;
	float3 Pos1		 : POSITION1;
	float2 TexUV0	 : TEXCOORD0;
	float2 TexUV1    : TEXCOORD1;	
	float3 Normal0   : NORMAL0;
	float3 Normal1   : NORMAL1;
	float4 Diffuse   : COLOR0;
	float4 Specular  : COLOR1;	

};

struct VS_OUTPUT
{
    float4 Pos       : POSITION;  
    float4 Diffuse   : COLOR0; 
    float4 Spec      : COLOR1;
    float2 TexUV     : TEXCOORD0;    
};


//--------------------------------------------------------------------------------------
// Vertex shader
//--------------------------------------------------------------------------------------
VS_OUTPUT VtxShader_Morphing( VS_INPUT In )
{
    VS_OUTPUT Out;
	
	Out.Pos = float4( lerp( In.Pos0, In.Pos1, weight ), 1.0f );
	Out.Pos = mul( Out.Pos, mtxWorld );	
	Out.Pos = mul( Out.Pos, mtxView );
	Out.Pos  = mul( Out.Pos, mtxProj );
	
	In.Normal0 = lerp( In.Normal0, In.Normal1, weight );
					
	float3 N = normalize( mul( In.Normal0, (float3x3)mtxWorld ) );
	float3 V = normalize( gViewVec - Out.Pos );
	float3 L = normalize( gLight );					
					
	// Specular lighting ----
	float3 R = reflect(L, N);
	R = normalize(R);

	float dpS = dot(V,R);
	dpS = max(0, dpS);
	Out.Spec = pow(dpS, 10);
	Out.Spec.a = 1.0f;
	// End Spec -------------
	
	// Diffuse lighting -----
	float dp = dot( -L, N );
	dp = max( 0, dp );
	
	Out.Diffuse = dp;
	Out.Diffuse.a = 1.0f;
	// End Diffuse ----------	
	
	// Pass-thru texc coordinates
	Out.TexUV = In.TexUV0;
		
    return Out;    
}

//--------------------------------------------------------------------------------------
// Pixel shader
//--------------------------------------------------------------------------------------

float4 PixShader_Morphing( VS_OUTPUT In ) : COLOR
{ 
	//float4 baseColour  = tex2D(baseTex0,  In.TexUV);
	float4 baseColour = { 1.0,0.2,0.0,1.0 };

	In.Diffuse = floor(In.Diffuse.x*35)/35.0f;
	In.Spec    = floor(In.Spec.x*35)/35.0f;

	baseColour *= In.Diffuse.x;
	
	return baseColour + In.Spec * 1.0f;
}

//---------------------------------------------------------------------------------------

technique MorphingEffect
{
    pass P0
    { 
        VertexShader = compile vs_2_0 VtxShader_Morphing( );
		PixelShader  = compile ps_2_0 PixShader_Morphing( );          
    }
}
//---------------------------------------------------------------------------------------