//--------------------------------------------------------------------------------------
// basic vertex and pixel shader

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

// transformation matrices
float4x4 mtxWorld;  
float4x4 mtxView;
float4x4 mtxLight;
float4x4 mtxProj;
float4   gLight;   
float3   vecView;  

texture baseTex;
texture depthTex;

sampler2D baseTex0 = sampler_state
{
	Texture = <baseTex>;
	MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler2D depthTex0 = sampler_state
{
	Texture = <depthTex>;
    MinFilter = Point;
    MagFilter = Point;
    Mipfilter = Point;
    AddressU = Clamp;
    AddressV = Clamp;
};

//--------------------------------------------------------------------------------------
// Vertex shader output structure
//--------------------------------------------------------------------------------------

struct VS_INPUT
{
	float4 Pos       : POSITION;
	float2 TexCoord0 : TEXCOORD0;
	float4 colour	 : COLOR0;	
	float3 Normal    : NORMAL;
};

struct VS_OUTPUT
{
    float4 Pos        : POSITION;  
    float2 TexUV0     : TEXCOORD0;
    float4 LightingVec: TEXCOORD1;
    float4 Diffuse    : COLOR0; 
    float4 Spec       : COLOR1;
};


//--------------------------------------------------------------------------------------
// Vertex shader
//--------------------------------------------------------------------------------------
VS_OUTPUT VtxShader( VS_INPUT In )
{
    VS_OUTPUT Out;

	Out.LightingVec = mul(In.Pos, mtxWorld);
	Out.LightingVec = mul(Out.LightingVec, mtxLight);	
	Out.LightingVec = mul(Out.LightingVec, mtxProj);
	
	// position xform	
	In.Pos   = mul(In.Pos, mtxWorld);
	In.Pos   = mul(In.Pos, mtxView);
	Out.Pos  = mul(In.Pos, mtxProj);
	
	float3 N = normalize( mul( In.Normal, (float3x3)mtxWorld ) );
	float3 V = normalize( vecView - Out.Pos );
	float3 L = normalize( gLight );
	
	// Specular lighting ----	
	float3 R = reflect( L, N );
	R = normalize( R );
	
	float dpS = dot( V,R );
	dpS = max( 0, dpS );
	Out.Spec = pow( dpS, 20 );
	Out.Spec.a = 1.0f;
	// End Spec -------------
	
	// Diffuse lighting -----
	float dp = dot( -L, N );
	dp = max( 0, dp );
	
	Out.Diffuse = dp;
	Out.Diffuse.a = 1.0f;	
	// End Diffuse ----------
	
	// pass-thru texc coordinates
	Out.TexUV0 = In.TexCoord0;
	//Out.LightingVec = normalize(Out.LightingVec);	
    return Out;    
}

//--------------------------------------------------------------------------------------
// Pixel shader
//--------------------------------------------------------------------------------------
float4 PixShader(VS_OUTPUT In) : COLOR
{ 
    float4 baseColour   = tex2D( baseTex0,  In.TexUV0 );
	
	float2 UV = float2(In.LightingVec.x, -In.LightingVec.y);
	UV /= In.LightingVec.w;
	UV = (UV + 1) / 2;
	
	float shadowDepth = tex2D(depthTex0, UV);
	float pixelDepth = In.LightingVec.z / In.LightingVec.w;

	float tmp = 1*In.Diffuse;
	if(tmp == 0)
		In.Diffuse = float4(0.2f,0.2f,0.2f,1.0f);
		
		
	baseColour *= In.Diffuse;
    baseColour += (In.Spec * 5.0f);
    
	//if ( UV.x > 1 || UV.y > 1 || UV.x < 0 || UV.y < 0 )
	//{
	//}
	//else
	//{
//		if(shadowDepth < pixelDepth)
//			baseColour *= 0.5f;
//	}
	if(shadowDepth < pixelDepth)
	{
		//baseColour*=0.5;
		baseColour = float4(1.0,1.0,1.0,1.0);
	}

	return baseColour;

}

//---------------------------------------------------------------------------------------

technique Shadow
{
    pass P0
    { 
        VertexShader = compile vs_2_0 VtxShader( );
		PixelShader  = compile ps_2_0 PixShader( );          
    }
}
//---------------------------------------------------------------------------------------